@@ -2,7 +2,7 @@
 
 # crosser.sh: Generic toolchain builder.
 #
-# (c) 2008-2010 Marko Lindqvist
+# (c) 2008-2011 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -103,7 +103,7 @@ else
     fi
   fi
   log_write 1 "No build steps defined, building everything"
-  STEPPARAM="native:sdl"
+  STEPPARAM="native:comp"
 fi
 
 . "$CROSSER_MAINDIR/steps/stepfuncs.sh"
@@ -207,7 +207,7 @@ build_glibc() {
   export CPPFLAGS=""
   export LDFLAGS=""
 
-  if ! build_generic "tgt-$1" "$2" "$CONFOPTIONS" "$4 install_root=$SYSPREFIX"
+  if ! LIBRARY_PATH="$CROSSER_NAT_LIBP" build_generic "tgt-$1" "$2" "$CONFOPTIONS" "$4 install_root=$SYSPREFIX"
   then
     return 1
   fi
@@ -336,8 +336,8 @@ kernel_setup() {
 
     log_write 3 "  Make params: $MAKEPARAMS"
 
-    if ! make $MAKEPARAMS \
-	           2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
+    if ! LIBRARY_PATH="$CROSSER_NAT_LIBP" C_INCLUDE_PATH="${CROSSER_NAT_INCP}" CPLUS_INCLUDE_PATH="${CROSSER_NAT_INCP}" \
+         make $MAKEPARAMS 2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
     then
       log_error "Kernel prepare failed"
       return 1
@@ -353,8 +353,8 @@ kernel_setup() {
     MAKEPARAMS="$CROSSPARAM $KERN_PARAM INSTALL_HDR_PATH=$SYSPREFIX/usr INSTALL_MOD_PATH=$SYSPREFIX $MAKETARGETS"
 
     log_write 3 "  Make params: $MAKEPARAMS"
-    if ! make $MAKEPARAMS \
-                2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
+    if ! LIBRARY_PATH="$CROSSER_NAT_LIBP" C_INCLUDE_PATH="${CROSSER_NAT_INCP}" CPLUS_INCLUDE_PATH="${CROSSER_NAT_INCP}" \
+         make $MAKEPARAMS 2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
     then
       if test "x$1" = "xfull"
       then
@@ -495,13 +495,13 @@ dummy_glibc_objects() {
 prepare_binutils_src() {
   if ! unpack_component binutils     $VERSION_BINUTILS              ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
-        (  patch_src binutils-$VERSION_BINUTILS binutils_makeinfo &&
-           cd "$CROSSER_SRCDIR/binutils-$VERSION_BINUTILS" && autoconf )) ||
+        (  patch_src binutils-$BASEVER_BINUTILS binutils_makeinfo &&
+           cd "$CROSSER_SRCDIR/binutils-$BASEVER_BINUTILS" && autoconf )) ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
-        (  patch_src binutils-$VERSION_BINUTILS binutils_printffix &&
-	   patch_src binutils-$VERSION_BINUTILS binutils_ignret ))  ||
+        (  patch_src binutils-$BASEVER_BINUTILS binutils_printffix &&
+	   patch_src binutils-$BASEVER_BINUTILS binutils_ignret ))  ||
      ! ( ! cmp_versions $VERSION_BINUTILS 2.20 ||
-         patch_src binutils-$VERSION_BINUTILS binutils_arm_empty_body )
+         patch_src binutils-$BASEVER_BINUTILS binutils_arm_empty_body )
   then
     log_error "Binutils setup failed"
     return 1
@@ -538,6 +538,24 @@ prepare_gcc_src() {
   fi
 }
 
+# Basic preparation of libgcrypt source tree
+#
+prepare_libgcrypt_src() {
+  if ! unpack_component libgcrypt $VERSION_LIBGCRYPT
+  then
+    log_error "Libgcrypt unpacking failed"
+    exit 1
+  fi
+
+  if ! patch_src libgcrypt-$VERSION_LIBGCRYPT gcrypt_gpgerr ||
+     ! autogen_component libgcrypt $VERSION_LIBGCRYPT       \
+       "aclocal automake autoconf"
+  then
+    log_error "Failed to prepare libgcrypt sources"
+    exit 1
+  fi
+}
+
 # Create link to intermediate build program to intermediate hostbin directory
 #
 # $1 - Program to link
@@ -612,6 +630,20 @@ then
   fi
 fi
 
+export PKG_CONFIG_PATH=$NATIVE_PREFIX/lib/pkgconfig
+
+# Workaround for debian (and derivatives) multiarch linker paths
+NO_VEND=$(. $CROSSER_MAINDIR/setups/native.sh ; echo $TMP_ARCH-$TMP_OS)
+if test "x$LIBRARY_PATH" = "x"
+then
+  CROSSER_NAT_LIBP="/usr/lib/$NO_VEND"
+else
+  CROSSER_NAT_LIBP="$LIBRARY_PATH:/usr/lib/$NO_VEND"
+fi
+CROSSER_NAT_INCP="/usr/include/$NO_VEND"
+
+BASEVER_BINUTILS="$(basever_binutils $VERSION_BINUTILS)"
+
 if test "x$STEP_NATIVE" = "xyes" ; then
   STEP="native"
   STEPADD="   "
@@ -630,30 +662,32 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      ! build_for_host   autoconf autoconf-$VERSION_AUTOCONF   ||
      ! unpack_component automake $VERSION_AUTOMAKE            ||
      ! build_for_host   automake automake-$VERSION_AUTOMAKE   ||
-     ! unpack_component Python   $VERSION_PYTHON              ||
-     ! build_for_host   Python   Python-$VERSION_PYTHON       ||
+     ! unpack_component Python3  $VERSION_PYTHON3             ||
+     ! build_for_host   Python3  Python-$VERSION_PYTHON3      ||
      ! unpack_component gtk-doc  $VERSION_GTK_DOC             ||
      ! build_for_host   gtk-doc  gtk-doc-$VERSION_GTK_DOC         \
        "--disable-scrollkeeper"                                   ||
+     ! unpack_component libffi     $VERSION_FFI                   ||
+     ! build_for_host   libffi     libffi-$VERSION_FFI            ||
+     ! unpack_component glib       $VERSION_GLIB                  ||
+     ! build_for_host   glib       glib-$VERSION_GLIB             ||
      ! unpack_component pkg-config $VERSION_PKG_CONFIG            ||
      ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
         patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
      ! build_for_host   pkg-config pkg-config-$VERSION_PKG_CONFIG ||
-     ! unpack_component glib       $VERSION_GLIB                  ||
-     ! build_for_host   glib       glib-$VERSION_GLIB             ||
      ! unpack_component          libgpg-error $VERSION_GPGERROR     ||
      ! build_for_host   libgpg-error libgpg-error-$VERSION_GPGERROR ||
-     ! unpack_component          libgcrypt $VERSION_LIBGCRYPT       ||
+     ! prepare_libgcrypt_src                                        ||
      ! build_for_host   libgcrypt libgcrypt-$VERSION_LIBGCRYPT      \
          "--disable-asm"                                            ||
      ! unpack_component libxslt    $VERSION_LIBXSLT               ||
      ! build_for_host   libxslt    libxslt-$VERSION_LIBXSLT       ||
      ! prepare_binutils_src                                   ||
-     ! build_for_host binutils binutils-$VERSION_BINUTILS     \
+     ! build_for_host binutils binutils-$BASEVER_BINUTILS     \
      "--with-tls --enable-stage1-languages=all"               ||
      ! prepare_gcc_src                                        ||
      ! log_write 1 "-Building native gcc can take hours!-"    ||
-     ! build_for_host gcc gcc-$VERSION_GCC                    \
+     ! LIBRARY_PATH="$CROSSER_NAT_LIBP" build_for_host gcc gcc-$VERSION_GCC                    \
      "--enable-languages=c,c++ --disable-multilib --with-tls"
   then
      crosser_error "Failed to build native compiler for host"
@@ -726,7 +760,7 @@ then
   export PATH="$PATH_CROSS"
   hash -r
 
-  if ! build_with_native_compiler binutils binutils-$VERSION_BINUTILS \
+  if ! build_with_native_compiler binutils binutils-$BASEVER_BINUTILS \
         "--with-sysroot=$SYSPREFIX --with-tls --enable-stage1-languages=all"
   then
     crosser_error "Binutils build failed"
@@ -916,12 +950,13 @@ fi   # STEP_CHAIN
 
 export CCACHE_DIR="$CROSSER_DST_PFX/.ccache"
 
+unset PKG_CONFIG_PATH
 export PKG_CONFIG_LIBDIR="$CROSSER_IM_PFX/lib/pkgconfig:$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
 
-PYTHON_SUBDIR="python$(echo $VERSION_PYTHON | sed 's/\./ /g' |
+PYTHON3_SUBDIR="python$(echo $VERSION_PYTHON3 | sed 's/\./ /g' |
 ( read MAJ MIN PATCH ; echo $MAJ.$MIN ))"
 
-export PYTHONPATH="$CROSSER_IM_PFX/lib/$PYTHON_SUBDIR/site-packages:$CROSSER_IM_PFX/lib/$PYTHON_SUBDIR/site-packages/xcbgen"
+export PYTHON_PATH="$CROSSER_IM_PFX/lib/$PYTHON3_SUBDIR/site-packages:$CROSSER_IM_PFX/lib/$PYTHON3_SUBDIR/site-packages/xcbgen"
 
 if test "x$STEP_BASELIB" = "xyes"
 then
@@ -963,7 +998,7 @@ then
     if test "x$STEP_NATIVE" != "xyes"
     then
       if ! unpack_component libgpg-error $VERSION_GPGERROR ||
-         ! unpack_component libgcrypt $VERSION_LIBGCRYPT   ||
+         ! prepare_libgcrypt_src                           ||
          ! unpack_component libxslt $VERSION_LIBXSLT
       then
         crosser_error "Unpacking of baselib source packages failed"
@@ -973,7 +1008,7 @@ then
 
     if ! unpack_component libxml2 $VERSION_LIBXML2                        ||
        ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       \
-         "--prefix=$CROSSER_IM_PFX" "" "/"
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                ||
        ! build_with_cross_compiler libgpg-error \
          libgpg-error-$VERSION_GPGERROR \
          "--prefix=$CROSSER_IM_PFX" "" "/"                                ||
@@ -1068,19 +1103,22 @@ then
     GLIB_VARS="$(read_configure_vars glib)"
     log_write 4 "Glib variables: $GLIB_VARS"
 
-    # If native step already executed, glib already unpacked
+    # If native step already executed, libffi and glib already unpacked
     if test "x$STEP_NATIVE" != "xyes"
     then
-      if ! unpack_component          glib          $VERSION_GLIB
+      if ! unpack_component          libffi        $VERSION_FFI  ||
+         ! unpack_component          glib          $VERSION_GLIB
       then
-        crosser_error "Glib unpacking failed"
+        crosser_error "Unpacking failed"
         exit 1
       fi
     fi
 
     STEP="gtk(im)"
     STEPADD="  "
-    if ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
+    if ! build_with_cross_compiler libffi        libffi-$VERSION_FFI                     \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                               ||
+       ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
          "--prefix=$CROSSER_IM_PFX $GLIB_VARS" "" "/"                                    ||
        ! unpack_component          freetype      $VERSION_FREETYPE                       ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
@@ -1104,7 +1142,8 @@ then
 
     STEP="gtk(tgt)"
     STEPADD=" "
-    if ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
+    if ! build_with_cross_compiler libffi        libffi-$VERSION_FFI                     ||
+       ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
          "--prefix=/usr $GLIB_VARS" ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              ||
        ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
@@ -1152,6 +1191,7 @@ then
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k26            ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k2628          ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k2636          ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_k2639                 ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_arm_outsb             ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_nostrip               ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_glinc                 ||
@@ -1192,6 +1232,31 @@ then
   fi
 fi
 
+if test "x$STEP_COMP" = "xyes"
+then
+  STEP="comp"
+  STEPADD="     "
+
+  # If native or chain step already executed, these preparations are done.
+  if test "x$STEP_NATIVE" != "xyes" && test "x$STEP_CHAIN" != "xyes"
+  then
+    if ! prepare_binutils_src ||
+       ! prepare_gcc_src
+    then
+      exit 1
+    fi
+  fi
+
+  if ! build_with_cross_compiler binutils binutils-$BASEVER_BINUTILS \
+       "--with-tls --enable-stage1-languages=all"                    ||
+     ! build_with_cross_compiler gcc gcc-$VERSION_GCC                \
+       "--disable-multilib --enable-languages=c --with-tls --enable-threads"
+  then
+    crosser_error "Target compiler build failed"
+    exit 1
+  fi
+fi
+
 if test "x$STEP_TEST" = "xyes"
 then
   STEP="test"
