@@ -82,23 +82,37 @@ build_component()
 # $3 - Extra configure options
 build_component_host()
 {
-  build_component_full "host-$1" "$1" "$2" "$3" "" "native"
+  if ! build_component_full "host-$1" "$1" "$2" "$3" "native"
+  then
+    BERR=true
+  else
+    BERR=false
+  fi
 
   # Reset command hash in case it already contained old version of the
   # just built tool
-  hash -r 
+  hash -r
+
+  if test "x$BERR" = "xtrue"
+  then
+    return 1
+  fi
 }
 
 # $1 - Build dir
 # $2 - Component
-# $3 - Version
+# $3 - Version, "0" to indicate that there isn't package to build after all
 # $4 - Extra configure options
-# $5 - Overwrite libtool ('overwrite')
-# $6 - Native ('native')
+# $5 - Native ('native')
 build_component_full()
 {
   log_packet "$1"
 
+  if test "x$3" = "x0"
+  then
+    return 0
+  fi
+
   SUBDIR="$(src_subdir $2 $3)"
 
   if test "x$SUBDIR" = "x"
@@ -113,10 +127,12 @@ build_component_full()
     log_error "Failed to create directory $BUILDDIR"
     return 1
   fi
+
+  (
   cd "$BUILDDIR"
   SRCDIR="$CROSSER_SRCDIR/$SUBDIR"
 
-  if test "x$6" != "xnative"
+  if test "x$5" != "xnative"
   then
     CONFOPTIONS="--prefix=$DLLSPREFIX --build=$BUILD --host=$TARGET --target=$TARGET $4"
     export CPPFLAGS="-isystem $DLLSPREFIX/include -isystem $TGT_HEADERS $TGT_MARCH $USER_CPPFLAGS"
@@ -140,18 +156,6 @@ build_component_full()
     fi
   fi
 
-  if test "x$5" = "xoverwrite" ; then
-    log_write 2 "Copying working libtool to $1"
-    if ! cp "$CROSSER_BUILDDIR/libtool/libtool" .
-    then
-      log_error "Failed to copy libtool"
-      return 1
-    fi
-  elif test "x$5" != "x" ; then
-    log_error "Illegal libtool overwrite parameter $6"
-    return 1
-  fi
-
   log_write 1 "Building $1"
   log_write 3 "  Make targets: [default] install"
 
@@ -166,6 +170,7 @@ build_component_full()
     log_error "Install for $1 failed"
     return 1
   fi
+  )
 }
 
 # Build zlib
@@ -190,6 +195,7 @@ build_zlib()
   export RANLIB=$TARGET-ranlib
   export AR=$TARGET-ar
 
+  (
   if ! cd "$CROSSER_SRCDIR/$SUBDIR"
   then
     log_error "Cannot change to directory $CROSSER_SRCDIR/$SUBDIR"
@@ -235,6 +241,7 @@ build_zlib()
     log_error "Failed to move libz dll:s to correct directory"
     return 1
   fi
+  )
 }
 
 # Build bzip2
@@ -254,6 +261,7 @@ build_bzip2()
     return 1
   fi
 
+  (
   if ! cd "$CROSSER_SRCDIR/$SUBDIR"
   then
     log_error "Cannot change to directory $CROSSER_SRCDIR/$SUBDIR"
@@ -283,6 +291,7 @@ build_bzip2()
     log_error "Install for $1 failed"
     return 1
   fi
+  )
 }
 
 # Update one autotools auxiliary file for component
@@ -437,61 +446,90 @@ then
   fi
 fi
 
-export PKG_CONFIG_LIBDIR="$DLLSPREFIX/lib/pkgconfig"
+export PKG_CONFIG_LIBDIR="$NATIVE_PREFIX/lib/pkgconfig"
 
 BASEVER_LIBTOOL="$(basever_libtool $VERSION_LIBTOOL)"
 GLIB_VARS="$(read_configure_vars glib)"
+GETTEXT_VARS="$(read_configure_vars gettext)"
 
 # glib_acsizeof -patch is required only when running autogen for glib
 if ! unpack_component     autoconf   $VERSION_AUTOCONF      ||
    ! build_component_host autoconf   $VERSION_AUTOCONF      ||
+   ! free_component       autoconf   $VERSION_AUTOCONF "host-autoconf" ||
    ! unpack_component     automake   $VERSION_AUTOMAKE      ||
    ! build_component_host automake   $VERSION_AUTOMAKE      ||
+   ! free_component       automake   $VERSION_AUTOMAKE "host-automake" ||
    ! unpack_component     libtool    $VERSION_LIBTOOL       ||
    ! build_component_host libtool    $BASEVER_LIBTOOL       ||
-   ! unpack_component     pkg-config $VERSION_PKG_CONFIG    ||
-   ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
-      patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
-   ! build_component_host pkg-config $VERSION_PKG_CONFIG    ||
+   ! free_build           "host-libtool"                    ||
+   ! unpack_component     libffi     $VERSION_FFI           ||
+   ! build_component_host libffi     $VERSION_FFI           ||
+   ! free_build           "host-libffi"                     ||
    ! unpack_component     glib       $VERSION_GLIB          ||
    ! (! cmp_versions $VERSION_GLIB 2.18.0 ||
         ( patch_src glib-$VERSION_GLIB glib_gmoddef  &&
           patch_src glib-$VERSION_GLIB glib_acsizeof &&
           autogen_component glib       $VERSION_GLIB \
            "libtoolize aclocal automake autoconf" ))        ||
-   ! build_component_host glib $VERSION_GLIB
+   ! build_component_host glib $VERSION_GLIB                ||
+   ! free_build           "host-glib"                       ||
+   ! unpack_component     pkg-config $VERSION_PKG_CONFIG    ||
+   ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
+      patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
+   ! build_component_host pkg-config $VERSION_PKG_CONFIG    ||
+   ! free_component       pkg-config $VERSION_PKG_CONFIG "host-pkg-config"
 then
   log_error "Native build failed"
   exit 1
 fi
 
-if ! unpack_component  libtool    $VERSION_LIBTOOL                   ||
-   ! build_component   libtool    $BASEVER_LIBTOOL                   ||
+export PKG_CONFIG_LIBDIR="$DLLSPREFIX/lib/pkgconfig"
+
+if ! build_component   libtool    $BASEVER_LIBTOOL                   ||
+   ! free_component    libtool    $BASEVER_LIBTOOL "libtool"         ||
    ! unpack_component  libiconv   $VERSION_ICONV                     ||
    ! build_component   libiconv   $VERSION_ICONV                     ||
+   ! free_component    libiconv   $VERSION_ICONV "libiconv"          ||
    ! unpack_component  zlib       $VERSION_ZLIB                      ||
    ! patch_src zlib               zlib_cctest                        ||
    ! patch_src zlib               zlib_seeko                         ||
    ! patch_src zlib               zlib_nolibc                        ||
    ! patch_src zlib               zlib_dllext                        ||
-   ! build_zlib        zlib                                          ||
+   ! build_zlib        zlib       $VERSION_ZLIB                      ||
+   ! free_src          zlib       $VERSION_ZLIB                      ||
    ! unpack_component  bzip2      $VERSION_BZIP2                     ||
    ! patch_src bzip2-$VERSION_BZIP2 bzip2_unhardcodecc               ||
    ! patch_src bzip2-$VERSION_BZIP2 bzip2_incpathsep                 ||
+   ! patch_src bzip2-$VERSION_BZIP2 bzip2_winapi                     ||
    ! build_bzip2       bzip2      $VERSION_BZIP2                     ||
+   ! free_src          bzip2      $VERSION_BZIP2                     ||
    ! unpack_component  curl       $VERSION_CURL                      ||
    ! build_component   curl       $VERSION_CURL                      ||
+   ! free_component    curl       $VERSION_CURL "curl"               ||
    ! unpack_component  libpng     $VERSION_PNG                       ||
    ! patch_src libpng-$VERSION_PNG png_symbol_prefix                 ||
    ! autogen_component libpng     $VERSION_PNG                       ||
    ! build_component   libpng     $VERSION_PNG                       ||
+   ! free_component    libpng     $VERSION_PNG "libpng"              ||
    ! unpack_component  gettext    $VERSION_GETTEXT                   ||
    ! ( is_minimum_version $VERSION_GETTEXT 0.18 ||
        ( patch_src gettext-$VERSION_GETTEXT gettext_bash &&
          patch_src gettext-$VERSION_GETTEXT gettext_no_rpl_optarg )) ||
-   ! (export LIBS="-liconv" && build_component gettext  $VERSION_GETTEXT) ||
+   ! ( is_max_version $VERSION_GETTEXT 0.17 ||
+       ( patch_src gettext-$VERSION_GETTEXT gettext_cxx_tools &&
+         ( cd "$CROSSER_SRCDIR/gettext-$VERSION_GETTEXT" &&
+           libtoolize &&
+           ./autogen.sh --quick --skip-gnulib ) \
+           >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
+     ))                                                               ||
+   ! (export LIBS="-liconv" && build_component gettext  $VERSION_GETTEXT \
+                               "$GETTEXT_VARS --enable-relocatable" ) ||
+   ! free_component    gettext    $VERSION_GETTEXT "gettext"          ||
+   ! build_component   libffi     $VERSION_FFI                        ||
+   ! free_component    libffi     $VERSION_FFI    "libffi"            ||
    ! build_component   glib       $VERSION_GLIB             \
-       "$GLIB_VARS"
+       "$GLIB_VARS"                                                   ||
+   ! free_component    glib       $VERSION_GLIB "glib"
 then
   log_error "Build failed"
   exit 1
@@ -499,12 +537,9 @@ fi
 
 if test "x$CROSSER_OPTION_JPEG" = "xon"
 then
-  if ! unpack_component jpeg $VERSION_JPEG "" "jpegsrc.v${VERSION_JPEG}"
-  then
-    log_error "Libjpeg download failed"
-    exit 1
-  fi
-  if ! build_component jpeg $VERSION_JPEG "--enable-shared"
+  if ! unpack_component jpeg $VERSION_JPEG "" "jpegsrc.v${VERSION_JPEG}" ||
+     ! build_component jpeg $VERSION_JPEG "--enable-shared"              ||
+     ! free_component jpeg $VERSION_JPEG "jpeg"
   then
     log_error "Libjpeg build failed"
     exit 1
@@ -514,7 +549,7 @@ else
   CONF_JPEG_GTK="--without-libjpeg"
 fi
 
-if is_minimum_version $VERSION_GTK 2.13.0
+if is_minimum_version $VERSION_GTK2 2.13.0
 then
   CONF_JPEG_GTK="$CONF_JPEG_GTK --without-libjasper"
 fi
@@ -525,7 +560,10 @@ then
   exit 1
 fi
 
-if ! patch_src tiff-$VERSION_TIFF tiff_config_headers
+if ! ( is_minimum_version $VERSION_TIFF 3.9.5 ||
+       patch_src tiff-$VERSION_TIFF tiff_config_headers ) ||
+   ! ( is_smaller_version $VERSION_TIFF 3.9.5 ||
+       patch_src tiff-$VERSION_TIFF tiff_config_headers_395 )
 then
   log_error "Tiff patching failed"
   exit 1
@@ -544,9 +582,11 @@ fi
 if ! ( is_minimum_version $VERSION_TIFF 3.9.0 ||
       autogen_component tiff       $VERSION_TIFF )                ||
    ! build_component_full                                         \
-     tiff tiff $VERSION_TIFF "$CONF_JPEG_TIFF" "overwrite"        ||
+     tiff tiff $VERSION_TIFF "$CONF_JPEG_TIFF"                    ||
+   ! free_component    tiff       $VERSION_TIFF "tiff"            ||
    ! unpack_component  expat      $VERSION_EXPAT                  ||
-   ! build_component   expat      $VERSION_EXPAT
+   ! build_component   expat      $VERSION_EXPAT                  ||
+   ! free_component    expat      $VERSION_EXPAT "expat"
 then
   log_error "Build failed"
   exit 1
@@ -557,7 +597,8 @@ if ! unpack_component  freetype   $VERSION_FREETYPE               ||
        patch_src freetype-$VERSION_FREETYPE freetype_dll )        ||
    ! ( is_minimum_version $VERSION_FREETYPE 2.3.6                 ||
        autogen_component freetype   $VERSION_FREETYPE )           ||
-   ! build_component   freetype   $VERSION_FREETYPE
+   ! build_component   freetype   $VERSION_FREETYPE               ||
+   ! free_component    freetype   $VERSION_FREETYPE "freetype"
 then
   log_error "Freetype build failed"
   exit 1
@@ -571,56 +612,81 @@ if ! unpack_component  fontconfig $VERSION_FONTCONFIG               ||
      "libtoolize aclocal automake autoconf"                             ||
    ! build_component   fontconfig $VERSION_FONTCONFIG                   \
      "--with-freetype-config=$DLLSPREFIX/bin/freetype-config --with-arch=$TARGET" ||
+   ! free_component    fontconfig $VERSION_FONTCONFIG "fontconfig" ||
    ! unpack_component  pixman     $VERSION_PIXMAN                 ||
    ! build_component   pixman     $VERSION_PIXMAN                 \
      "--disable-gtk"                                              ||
+   ! free_component    pixman     $VERSION_PIXMAN "pixman"        ||
    ! unpack_component  cairo      $VERSION_CAIRO                  ||
    ! rm -f "$CROSSER_SRCDIR/cairo-$VERSION_CAIRO/src/cairo-features.h" ||
    ! ( is_smaller_version $VERSION_CAIRO 1.10.0 ||
        patch_src         cairo-$VERSION_CAIRO cairo_ffs )         ||
    ! build_component   cairo      $VERSION_CAIRO                  \
      "--disable-xlib --enable-win32"                              ||
+   ! free_component    cairo      $VERSION_CAIRO "cairo"          ||
    ! unpack_component  pango      $VERSION_PANGO                  ||
    ! CXX="$TARGET-g++" build_component   pango      $VERSION_PANGO                  ||
+   ! free_component    pango      $VERSION_PANGO "pango"          ||
    ! unpack_component  atk        $VERSION_ATK                    ||
    ! ( is_smaller_version $VERSION_ATK     1.24.0  ||
        patch_src          atk-$VERSION_ATK atk_def    )           ||
    ! autogen_component atk        $VERSION_ATK                    \
      "libtoolize aclocal automake autoconf"                       ||
-   ! build_component   atk        $VERSION_ATK
+   ! build_component   atk        $VERSION_ATK                    ||
+   ! free_component    atk        $VERSION_ATK "atk"
 then
   log_error "Build failed"
   exit 1
 fi
 
-if ! ( is_smaller_version $VERSION_GTK 2.22.0 ||
+if ! ( is_smaller_version $VERSION_GTK2 2.22.0 ||
        ( unpack_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
-         build_component gdk-pixbuf $VERSION_GDK_PIXBUF ))        ||
-   ! unpack_component  gtk+       $VERSION_GTK                    ||
-   ! ( is_minimum_version $VERSION_GTK      2.12.10 ||
-       patch_src gtk+-$VERSION_GTK          gtk_blddir )          ||
-   ! ( is_minimum_version $VERSION_GTK      2.13.2 ||
-       patch_src gtk+-$VERSION_GTK          gtk_check_cxx )       ||
-   ! ( is_smaller_version $VERSION_GTK      2.14.0 ||
-       is_minimum_version $VERSION_GTK      2.16.0 ||
-       patch_src gtk+-$VERSION_GTK          gtk_gailutildef )     ||
-   ! ( is_minimum_version $VERSION_GTK      2.16.0 ||
-       autogen_component gtk+       $VERSION_GTK   \
+         ( is_smaller_version $VERSION_GTK2 2.24.0 ||
+           patch_src gdk-pixbuf-$VERSION_GDK_PIXBUF gdkpixbuf_gdiplusdef ) &&
+         build_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
+         free_component  gdk-pixbuf $VERSION_GDK_PIXBUF "gdk-pixbuf")) ||
+   ! unpack_component  gtk2       $VERSION_GTK2                   ||
+   ! ( is_minimum_version $VERSION_GTK2     2.12.10 ||
+       patch_src gtk+-$VERSION_GTK2         gtk_blddir )          ||
+   ! ( is_minimum_version $VERSION_GTK2     2.13.2 ||
+       patch_src gtk+-$VERSION_GTK2         gtk_check_cxx )       ||
+   ! ( is_smaller_version $VERSION_GTK2     2.14.0 ||
+       is_minimum_version $VERSION_GTK2     2.16.0 ||
+       patch_src gtk+-$VERSION_GTK2         gtk_gailutildef )     ||
+   ! ( is_minimum_version $VERSION_GTK2     2.16.0 ||
+       autogen_component gtk+       $VERSION_GTK2   \
          "libtoolize aclocal automake autoconf" )                 ||
-   ! build_component   gtk+       $VERSION_GTK                    \
+   ! build_component_full gtk2 gtk+ $VERSION_GTK2                 \
      "--disable-cups --disable-explicit-deps $CONF_JPEG_GTK"      ||
+   ! free_component   gtk+        $VERSION_GTK2 "gtk2"            ||
+   ! unpack_component gtk3        $VERSION_GTK3                   ||
+   ! rm -f $CROSSER_SRCDIR/gtk+-$VERSION_GTK3/gdk/gdkconfig.h     ||
+   ! ( is_smaller_version "$VERSION_GTK3" 3.0.0 ||
+       patch_src gtk+-$VERSION_GTK3 gtk3_marshalers )             ||
+   ! build_component_full gtk3 gtk+ $VERSION_GTK3                 ||
+   ! free_component   gtk+        $VERSION_GTK3 "gtk3"            ||
    ! unpack_component gtk-engines $VERSION_GTK_ENG                ||
-   ! build_component  gtk-engines $VERSION_GTK_ENG
+   ! build_component  gtk-engines $VERSION_GTK_ENG                ||
+   ! free_component   gtk-engines $VERSION_GTK_ENG "gtk-engines"
 then
   log_error "gtk+ stack build failed"
   exit 1
 fi
 
+if is_minimum_version $VERSION_GDK_PIXBUF 2.22.0
+then
+  GDKPBL="lib/gdk-pixbuf-2.0/2.10.0/loaders.cache"
+else
+  GDKPBL="etc/gtk-2.0/gdk-pixbuf.loaders"
+fi
+
+WGDKPBL="$(echo $GDKPBL | sed 's,/,\\,g')"
+
 if test "x$AUTOWINE" = "xyes" ; then
   log_write 1 "Creating configuration files"
   if ! mkdir -p $DLLSPREFIX/etc/pango ||
      ! $DLLSPREFIX/bin/pango-querymodules.exe > $DLLSPREFIX/etc/pango/pango.modules ||
-     ! $DLLSPREFIX/bin/gdk-pixbuf-query-loaders.exe > $DLLSPREFIX/etc/gtk-2.0/gdk-pixbuf.loaders
+     ! $DLLSPREFIX/bin/gdk-pixbuf-query-loaders.exe > $DLLSPREFIX/$GDKPBL
   then
     log_error "Failed to create configuration files in wine."
     exit 1
@@ -630,17 +696,20 @@ log_write 1 "Creating setup.bat"
 (
   echo -n -e "if not exist etc\pango mkdir etc\pango\r\n"
   echo -n -e "bin\pango-querymodules.exe > etc\pango\pango.modules\r\n"
-  echo -n -e "bin\gdk-pixbuf-query-loaders.exe > etc\gtk-2.0\gdk-pixbuf.loaders\r\n"
+  echo -n -e "bin\gdk-pixbuf-query-loaders.exe > $WGDKPBL\r\n"
 ) > "$DLLSPREFIX/setup.bat"
 log_write 1 "IMPORTANT: Remember to create configuration files when installing to target"
 
 if ! unpack_component  SDL        $VERSION_SDL          ||
    ! build_component   SDL        $VERSION_SDL          ||
+   ! free_component    SDL        $VERSION_SDL "SDL"    ||
    ! unpack_component  SDL_image  $VERSION_SDL_IMAGE    ||
    ! build_component   SDL_image  $VERSION_SDL_IMAGE    ||
+   ! free_component    SDL_image  $VERSION_SDL_IMAGE "SDL_image" ||
    ! unpack_component  SDL_mixer  $VERSION_SDL_MIXER    ||
    ! build_component   SDL_mixer  $VERSION_SDL_MIXER    \
-     "--disable-music-mp3 --disable-smpegtest"
+     "--disable-music-mp3 --disable-smpegtest"          ||
+   ! free_component    SDL_mixer  $VERSION_SDL_MIXER "SDL_mixer"
 then
   log_error "SDL stack build failed"
   exit 1
