@@ -35,6 +35,14 @@ download_file() {
       APPEND=yes
     fi
   else
+    # Creating new filelist
+    if test "x$CROSSER_GROUP" != "x" ; then
+      touch filelist.txt
+      if ! chown ":$CROSSER_GROUP" filelist.txt ; then
+        echo "Cannot set owner group for filelist.txt" >&2
+        return 1
+      fi
+    fi
     APPEND=yes
   fi
 
@@ -54,21 +62,36 @@ download_file() {
     return 1
   fi
 
+  if test "x$CROSSER_GROUP" != "x" ; then
+    if ! chown ":$CROSSER_GROUP" "$2" ; then
+      echo "Cannot set owner group for \"$2\"" >&2
+      return 1
+    fi
+  fi
+
   echo "Downloaded $2"
 }
 
 # $1 - Base URL
-# $2 - Base filename
+# $2 - Base name, usually base filename
 # $3 - Version      - or nonstandard filename (See $4)
 # $4 - Package type - empty means nonstandard naming (See $3)
 download_packet() {
 
+  if test "x$2" = "xgtk2" || test "x$2" = "xgtk3" ; then
+    BFNAME="gtk+"
+  elif test "x$2" = "xPython3" || test "x$2" = "xPython2" ; then
+    BFNAME="Python"
+  else
+    BFNAME="$2"
+  fi
+
   if test "x$4" = "xdsc" ; then
-    DLFILENAME="${2}_$3.$4"
+    DLFILENAME="${BFNAME}_$3.$4"
 
     if ! download_file "$1" "$DLFILENAME"
     then
-      echo "Download of $2 version $3 dsc file failed" >&2
+      echo "Download of $BFNAME version $3 dsc file failed" >&2
       return 1
     fi
 
@@ -84,7 +107,7 @@ download_packet() {
           else
             if ! download_file "$1" "$PART3"
             then
-              echo "Download of $2 version $3 file $PART3 failed" >&2
+              echo "Download of $BFNAME version $3 file $PART3 failed" >&2
               return 1
             fi
           fi
@@ -98,11 +121,11 @@ download_packet() {
     if test "x$4" = "x" ; then
       DLFILENAME="$3"
     else
-      DLFILENAME="$2-$3.$4"
+      DLFILENAME="$BFNAME-$3.$4"
     fi
     if ! download_file "$1" "$DLFILENAME"
     then
-       echo "Download of $2 version $3 failed" >&2
+       echo "Download of $BFNAME version $3 failed" >&2
        return 1
     fi
   fi
@@ -367,7 +390,8 @@ then
     glib)        VERSION_GLIB=$VERSION_SELECTED ;;
     pango)       VERSION_PANGO=$VERSION_SELECTED ;;
     gdk-pixbuf)  VERSION_GDK_PIXBUF=$VERSION_SELECTED ;;
-    gtk+)        VERSION_GTK=$VERSION_SELECTED ;;
+    gtk2)        VERSION_GTK2=$VERSION_SELECTED ;;
+    gtk3)        VERSION_GTK3=$VERSION_SELECTED ;;
     gtk-engines) VERSION_GTK_ENG=$VERSION_SELECTED ;;
     gtk-doc)     VERSION_GTK_DOC=$VERSION_SELECTED ;;
     atk)         VERSION_ATK=$VERSION_SELECTED ;;
@@ -378,19 +402,22 @@ then
     automake)    VERSION_AUTOMAKE=$VERSION_SELECTED ;;
     libtool)     VERSION_LIBTOOL=$VERSION_SELECTED ;;
     mpfr)        VERSION_MPFR=$VERSION_SELECTED ;;
-    Python)      VERSION_PYTHON=$VERSION_SELECTED ;;
+    Python3)     VERSION_PYTHON3=$VERSION_SELECTED ;;
+    Python2)     VERSION_PYTHON2=$VERSION_SELECTED ;;
     jpeg)        VERSION_JPEG=$VERSION_SELECTED ;;
     xproto)      VERSION_XORG_XPROTO=$VERSION_SELECTED ;;
     xextproto)   VERSION_XORG_XEXTPROTO=$VERSION_SELECTED ;;
     xtrans)      VERSION_XORG_XTRANS=$VERSION_SELECTED ;;
     libXau)      VERSION_XORG_LIBXAU=$VERSION_SELECTED ;;
     libX11)      VERSION_XORG_LIBX11=$VERSION_SELECTED ;;
+    linux)       VERSION_KERNEL=$VERSION_SELECTED ;;
   esac
 fi
 GLIB_DIR="$(echo $VERSION_GLIB | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 PANGO_DIR="$(echo $VERSION_PANGO | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GDK_PB_DIR="$(echo $VERSION_GDK_PIXBUF | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
-GTK_DIR="$(echo $VERSION_GTK | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+GTK2_DIR="$(echo $VERSION_GTK2 | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+GTK3_DIR="$(echo $VERSION_GTK3 | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GTK_ENG_DIR="$(echo $VERSION_GTK_ENG | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GTK_DOC_DIR="$(echo $VERSION_GTK_DOC | sed 's/\./ /g' | (read MAJOR MINOR ; echo -n $MAJOR.$MINOR ))"
 ATK_DIR="$(echo $VERSION_ATK | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
@@ -446,6 +473,8 @@ else
   LIBTOOL_PACK="tar.bz2"
 fi
 
+KERNEL_DIR="v$(echo $VERSION_KERNEL | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+
 download_needed "$MIRROR_GNU/libtool/"  "libtool"  "$VERSION_LIBTOOL"  $LIBTOOL_PACK
 
 RET="$?"  
@@ -457,7 +486,9 @@ download_needed "$MIRROR_GNU/autoconf/" "autoconf" "$VERSION_AUTOCONF" "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_GNU/automake/" "automake" "$VERSION_AUTOMAKE" "$AUTOMAKE_PACK"
 RET="$RET $?"
-download_needed "http://www.python.org/ftp/python/$VERSION_PYTHON/" "Python" "$VERSION_PYTHON" "tgz"
+download_needed "http://www.python.org/ftp/python/$VERSION_PYTHON3/" "Python3" "$VERSION_PYTHON3" "tgz"
+RET="$RET $?"
+download_needed "http://www.python.org/ftp/python/$VERSION_PYTHON2/" "Python2" "$VERSION_PYTHON2" "tgz"
 RET="$RET $?"
 download_needed "http://pkgconfig.freedesktop.org/releases/" "pkg-config" "$VERSION_PKG_CONFIG" "tar.gz"
 RET="$RET $?"
@@ -469,7 +500,7 @@ download_needed "$MIRROR_GNU/glibc/"    "glibc-ports" "$VERSION_GLIBC_PORTS" "ta
 RET="$RET $?"
 download_needed "$MIRROR_DEB/pool/main/e/eglibc/"       "eglibc"     "$VERSION_EGLIBC_DEB" "dsc"
 RET="$RET $?"
-download_needed "$MIRROR_KERNEL/pub/linux/kernel/v2.6/" "linux" "$VERSION_KERNEL"  "tar.bz2"
+download_needed "$MIRROR_KERNEL/pub/linux/kernel/$KERNEL_DIR/" "linux" "$VERSION_KERNEL"  "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_SOURCEWARE/pub/newlib/"        "newlib" "$VERSION_NEWLIB" "tar.gz"
 RET="$RET $?"
@@ -509,7 +540,9 @@ download_needed "$MIRROR_SAVANNAH/releases/freetype/"   "freetype"   "$VERSION_F
 RET="$RET $?"
 download_needed "http://fontconfig.org/release/"        "fontconfig" "$VERSION_FONTCONFIG" "tar.gz"
 RET="$RET $?"
-download_needed "http://curl.haxx.se/download/"         "curl"       "$VERSION_CULR" "tar.bz2"
+download_needed "http://curl.haxx.se/download/"         "curl"       "$VERSION_CURL"       "tar.bz2"
+RET="$RET $?"
+download_needed "ftp://sourceware.org/pub/libffi/"      "libffi"     "$VERSION_FFI"        "tar.gz"
 RET="$RET $?"
 download_needed "http://cairographics.org/releases/"    "pixman"     "$VERSION_PIXMAN"     "tar.gz"
 RET="$RET $?"
@@ -523,7 +556,9 @@ download_needed "$MIRROR_GNOME/sources/gtk-doc/$GTK_DOC_DIR/" "gtk-doc" "$VERSIO
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gdk-pixbuf/$GDK_PB_DIR/" "gdk-pixbuf" "$VERSION_GDK_PIXBUF"  "tar.bz2" 
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/gtk+/$GTK_DIR/"  "gtk+"       "$VERSION_GTK"        "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/gtk+/$GTK2_DIR/" "gtk2"       "$VERSION_GTK2"        "tar.bz2"
+RET="$RET $?"
+download_needed "$MIRROR_GNOME/sources/gtk+/$GTK3_DIR/" "gtk3"       "$VERSION_GTK3"        "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk-engines/$GTK_ENG_DIR/"  "gtk-engines" "$VERSION_GTK_ENG" "tar.bz2"
 RET="$RET $?"
