@@ -2,7 +2,7 @@
 
 # helpers.sh: Functions for Crosser
 #
-# (c) 2008-2010 Marko Lindqvist
+# (c) 2008-2011 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -95,7 +95,7 @@ log_error() {
 # $1 - Error message
 crosser_error() {
   log_error "$1"
-  if test "x$DEBUG_CROSSER" = "xyes"
+  if test "x$CROSSER_DEBUG" = "xyes"
   then
      ( echo "** Env dump begins **"
        /usr/bin/env
@@ -195,29 +195,45 @@ upstream_patch() {
 # Unpack component package to source directory
 #
 # $1   - Package name
-# $2   - Package version
+# $2   - Package version, "0" to indicate that there is no package after all
 # [$3] - Subdir in source hierarchy
 # [$4] - Package file name base in case it's not 'name-version'
 # [$5] - Number of patches
 unpack_component() {
-  if test "x$CROSSER_DOWNLOAD" = "xdemand" ; then
-    log_write 1 "Fetching $1 version $2"
+  if test "x$1" = "xgtk2" || test "x$1" = "xgtk3"
+  then
+    BNAME="gtk+"
+  elif test "x$1" = "xPython3" || test "x$1" = "xPython2"
+  then
+    BNAME="Python"
+  else
+    BNAME="$1"
+  fi
+
+  if test "x$2" = "x0"
+  then
+    return 0
+  fi
+
+  if test "x$CROSSER_DOWNLOAD" = "xdemand"
+  then
+    log_write 1 "Fetching $BNAME version $2"
     if ! ( cd "$PACKETDIR" && "$CROSSER_MAINDIR/scripts/download_packets.sh" --packet "$1" "$2" "$5" \
          >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log" )
     then
-      log_error "Failed to download $1 version $2"
+      log_error "Failed to download $BNAME version $2"
       return 1
     fi
   fi
 
-  log_write 1 "Unpacking $1 version $2"
+  log_write 1 "Unpacking $BNAME version $2"
 
   if test "x$4" != "x"
   then
     # Custom file name format
     NAME_BASE="$4"
   else
-    NAME_BASE="$1-$2"
+    NAME_BASE="$BNAME-$2"
   fi
 
   if test -e "$PACKETDIR/$NAME_BASE.tar.bz2" ; then
@@ -238,28 +254,86 @@ unpack_component() {
       log_error "Unpacking $NAME_BASE.tgz failed"
       return 1
     fi
-  elif test -e "$PACKETDIR/${1}_${2}.dsc" ; then
+  elif test -e "$PACKETDIR/${BNAME}_${2}.dsc" ; then
     if ! which dpkg-source >/dev/null ; then
       log_error "No way to unpack debian source packages"
       return 1
     fi
     if test "x$3" = "x" ; then
-      SRCDIR="$1"
+      SRCDIR="$BNAME"
     else
       SRCDIR="$3"
     fi
-    if ! dpkg-source -x "$PACKETDIR/${1}_${2}.dsc" "$CROSSER_SRCDIR/$SRCDIR" \
+    if ! dpkg-source -x "$PACKETDIR/${BNAME}_${2}.dsc" "$CROSSER_SRCDIR/$SRCDIR" \
                      >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
     then
-      log_error "Unpacking $1_$2.dsc failed"
+      log_error "Unpacking $BNAME_$2.dsc failed"
       return 1
     fi
   else
-    log_error "Can't find $1 version $2 package to unpack."
+    log_error "Can't find $BNAME version $2 package to unpack."
     return 1
   fi
 }
 
+# Free components temporary directories
+#
+# $1 -   Package
+# $2 -   Version
+# $3 -   Builddir
+free_component() {
+  free_src "$1" "$2"
+  free_build "$3"
+}
+
+# Free components temporary source directory
+#
+# $1 -   Package
+# $2 -   Version
+free_src() {
+  if test "x$CROSSER_TMPFREE" != "xyes"
+  then
+    return 0
+  fi
+
+  log_write 2 "Free source for $1 version $2"
+
+  SRCSUBDIR=$(src_subdir $1 $2)
+  if test "x$SRCSUBDIR" = "x"
+  then
+    echo "Cannot find srcdir of $1 version $2 to remove" >&2
+    return 1
+  fi
+
+  rm -Rf "$CROSSER_SRCDIR/$SRCSUBDIR"
+}
+
+# Free components temporary build directory
+#
+# $1 -   Builddir
+free_build() {
+  if test "x$CROSSER_TMPFREE" != "xyes" 
+  then
+    return 0
+  fi
+
+  if test "x$1" = "x"
+  then
+    echo "No builddir given for free_build()" >&2
+    return 1
+  fi
+
+  log_write 2 "Free builddir $1"
+
+  if ! test -d "$CROSSER_BUILDDIR/$1"
+  then
+    echo "Cannot find builddir \"$CROSSER_BUILDDIR/$1\" to remove" >&2
+    return 1
+  fi
+
+  rm -Rf "$CROSSER_BUILDDIR/$1"
+}
+
 # Output subdir under source hierarchy where component lives
 #
 # $1 -   Package
@@ -674,13 +748,13 @@ write_crosser_env() {
   fi
 }
 
-# Prints configure variables for component
+# Prints configure variables for component with possible extra space in end
 #
 # $1 - Component
 #
 # 0 - Success
 # 1 - Failure
-read_configure_vars() {
+read_configure_vars_sub() {
   CONF_FILE="$CROSSER_MAINDIR/setups/cachevars/$1.vars"
 
   if test -f "$CONF_FILE"
@@ -702,6 +776,23 @@ read_configure_vars() {
   return 0
 }
 
+# Prints configure variables for component
+#
+# $1 - Component
+#
+# 0 - Success
+# 1 - Failure
+read_configure_vars() {
+  FULLTEXT="$(read_configure_vars_sub "$1")"
+
+  if test "$?" = "1"
+  then
+    return 1
+  fi
+
+  echo "$FULLTEXT" | sed 's/ $//' 
+}
+
 # Check if packet directory exist and possibly create one
 #
 # 0 - Packetdir exist
